package com.me.brickbuster.state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.me.brickbuster.BrickBuster;
import com.me.brickbuster.entity.Ball;
import com.me.brickbuster.entity.Brick;
import com.me.brickbuster.entity.Paddle;
import com.me.brickbuster.entity.powerup.*;

import java.util.*;

public class PlayState extends State {

	public static final int SHIELD_HEIGHT = 30;
	public static final float POWERUP_CHANCE = 0.15f;

	public static final int COLUMNS = 9;
	public static final int ROWS = 8;

	public List<PowerUp> powerUps;
	public Paddle paddle;
	public List<Ball> balls;
	public List<Brick> bricks;

	private int shieldCount = 0;
	private float updateTime = 0f;

	public PlayState(BrickBuster game) {
		super(game);
	}

	@Override
	public void setup() {
		powerUps = new ArrayList<PowerUp>();
		paddle = new Paddle(this);

		int brick_padding = (BrickBuster.BOARD_WIDTH - COLUMNS * Brick.BRICK_WIDTH) / (COLUMNS + 1);
		bricks = new ArrayList<Brick>();
		for (int col = 0; col < COLUMNS; col++) {
			for (int row = 0; row < ROWS; row++) {
				int x = brick_padding + (col * (Brick.BRICK_WIDTH + brick_padding));
				int y = brick_padding + Brick.BRICK_HEIGHT + (row * (Brick.BRICK_HEIGHT + brick_padding));

				PowerUpType powerUpType = null;
				if (MathUtils.randomBoolean(POWERUP_CHANCE)) {
					powerUpType = PowerUpType.getWeightedRandom();
				}

				bricks.add(new Brick(this, powerUpType, x, BrickBuster.BOARD_HEIGHT - y));
			}
		}

		balls = new ArrayList<Ball>();
		balls.add(new Ball(this));
	}

	@Override
	public void render() {
		long start = System.nanoTime();
		for (Brick block : bricks) {
			block.render(game.sr);
		}
		for (PowerUp powerUp : powerUps) {
			powerUp.render(game.sr);
		}
		for (Ball ball : balls) {
			ball.render(game.sr);
		}
		paddle.render(game.sr);

		if (getShieldCount() > 0) {
			game.sr.begin(ShapeType.Filled);
			game.sr.setColor(Color.SALMON);
			game.sr.rect(0, 0, BrickBuster.BOARD_WIDTH, getShieldCount() * SHIELD_HEIGHT);
			game.sr.end();
		}
		long renderTime = System.nanoTime() - start;

		game.sb.begin();
		game.font.setColor(Color.GRAY);
		game.font.draw(game.sb, String.format("FPS: %d Update: %.2f ms Render: %.2f ms",
				Gdx.graphics.getFramesPerSecond(), updateTime/1000000f, renderTime/1000000f), 10, BrickBuster.BOARD_HEIGHT-10);
		game.sb.end();
	}

	@Override
	public void update(float dt) {
		long start = System.nanoTime();
		paddle.update(dt);

		for (Iterator<Ball> it = balls.iterator(); it.hasNext();) {
			Ball ball = it.next();
			ball.update(dt);
			if (ball.isDead()) {
				it.remove();
			}
		}
		if (balls.isEmpty()) {
			ballReset();
		}

		for (Iterator<PowerUp> it = powerUps.iterator(); it.hasNext();) {
			PowerUp powerUp = it.next();
			powerUp.update(dt);
			if(powerUp.isCaught()) {
				it.remove();
			}
		}

		if (bricks.isEmpty()) {
			game.setScreen(new MenuState(game));
			dispose();
		}
		updateTime = System.nanoTime() - start;
	}

	@Override
	public void dispose() {
		super.dispose();

		powerUps.clear();
		powerUps = null;
		balls.clear();
		balls = null;
		bricks.clear();
		bricks = null;
		paddle = null;
	}

	public int getShieldCount() {
		return shieldCount;
	}

	public void addShield() {
		shieldCount++;
		paddle.setY(paddle.getY() + SHIELD_HEIGHT);
		for (Ball ball : balls) {
			if (ball.isStuck()) {
				ball.setY(ball.getY() + SHIELD_HEIGHT);
			}
		}
	}

	public void removeShield() {
		shieldCount--;
		paddle.setY(paddle.getY() - SHIELD_HEIGHT);
		for (Ball ball : balls) {
			if (ball.isStuck()) {
				ball.setY(ball.getY() - SHIELD_HEIGHT);
			}
		}
	}

	public void ballReset() {
		Ball ball = new Ball(this);
		ball.setX(paddle.getX());
		ball.setY(Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + Ball.RADIUS);
		balls.add(ball);
		paddle.setSticky(false);
		paddle.setWidth(Paddle.DEFAULT_WIDTH);
	}

}
